package org.jvnet.hifaces.application.navigation.impl;

import java.io.IOException;

import javax.faces.FacesException;
import javax.faces.application.ViewHandler;
import javax.faces.component.UIViewRoot;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.servlet.ServletRequest;

import org.jvnet.hifaces.application.navigation.Navigator;
import org.jvnet.hifaces.application.navigation.impl.State.Transition;

public class DefaultNavigator implements Navigator {

	public static final String RELOAD_OUTCOME = "<reload>";

	public static final String NEXT_OUTCOME = "<next>";

	public static final String PREVIOUS_OUTCOME = "<previous>";

	public static final String RELOAD_VIEW_ID = "<reload>";

	public static final String NEXT_VIEW_ID = "<next>";

	public static final String PREVIOUS_VIEW_ID = "<previous>";

	public static final String CURRENT_STATE_ID_REQUEST_PARAMETER_NAME = "hicsid";

	public static final String NEXT_STATE_ID_REQUEST_PARAMETER_NAME = "hinsid";

	public static final String CURRENT_STATE_REQUEST_ATTRIBUTE_NAME = "_hiCurrentState";

	private static final long serialVersionUID = 1L;

	public void navigate(final FacesContext facesContext, String fromAction,
			String fromOutcome, String toViewId, boolean redirect) {

		final ServletRequest request = (ServletRequest) facesContext
				.getExternalContext().getRequest();

		final State currentState = (State) request
				.getAttribute(CURRENT_STATE_REQUEST_ATTRIBUTE_NAME);

		if (PREVIOUS_VIEW_ID.equals(toViewId)) {
			if (currentState.isPreviousAvailable()) {
				navigate(facesContext, currentState.getPreviousTransition());
			}
		} else if (NEXT_VIEW_ID.equals(toViewId)) {
			if (currentState.isNextAvailable()) {
				navigate(facesContext, currentState.getNextTransition());
			}
		} else if (RELOAD_VIEW_ID.equals(toViewId)) {
		} else if (toViewId.equals(currentState.getViewId())) {
			// Nothing to do
		} else {
			final State nextState = State.createState(facesContext, toViewId);

			if (redirect) {
				currentState.redirectNext(nextState);
			} else {
				currentState.forwardNext(nextState);
			}

			navigate(facesContext, currentState.getNextTransition());
		}
	}

	private void navigate(FacesContext facesContext, Transition transition) {

		final ViewHandler viewHandler = facesContext.getApplication()
				.getViewHandler();

		final State from = transition.getFrom();
		final State to = transition.getTo();
		final String toViewId = to.getViewId();

		if (!transition.isRedirect()) {
			final UIViewRoot viewRoot = viewHandler.createView(facesContext,
					toViewId);
			facesContext.setViewRoot(viewRoot);
			facesContext.renderResponse();
		} else {

			final ExternalContext externalContext = facesContext
					.getExternalContext();
			final String location = viewHandler.getActionURL(facesContext,
					toViewId);
			final String delimiter = location.contains("?") ? "&" : "?";

			final String newLocation = location + delimiter
					+ CURRENT_STATE_ID_REQUEST_PARAMETER_NAME + "="
					+ from.getId() + "&" + NEXT_STATE_ID_REQUEST_PARAMETER_NAME
					+ "=" + to.getId();

			try {
				externalContext.redirect(newLocation);
			} catch (IOException e) {
				throw new FacesException(e.getMessage(), e);
			}
		}
	}

	public void reload(FacesContext facesContext, String fromAction,
			String fromOutcome) {
	}
}
