package org.jvnet.hifaces.convert;

import java.text.MessageFormat;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;

public class MessageUtils {
	private static final String DETAIL_SUFFIX = "_detail";

	public static void addErrorMessage(UIComponent component, String messageId,
			Object args[]) {

		final FacesContext context = FacesContext.getCurrentInstance();
		final FacesMessage facesMessage = MessageUtils.getErrorMessage(context,
				messageId, new Object[] {});
		context.addMessage(component.getClientId(context), facesMessage);
	}

	public static FacesMessage getErrorMessage(FacesContext facesContext,
			String messageId, Object args[]) {
		return getMessage(facesContext, facesContext.getViewRoot().getLocale(),
				FacesMessage.SEVERITY_ERROR, messageId, args);
	}

	public static FacesMessage getMessage(FacesContext facesContext,
			Locale locale, FacesMessage.Severity severity, String messageId,
			Object args[]) {
		ResourceBundle appBundle;
		ResourceBundle defBundle;
		String summary;
		String detail;

		appBundle = getApplicationBundle(facesContext, locale);
		summary = getBundleString(appBundle, messageId);
		if (summary != null) {
			detail = getBundleString(appBundle, messageId + DETAIL_SUFFIX);
		} else {
			defBundle = getDefaultBundle(facesContext, locale);
			summary = getBundleString(defBundle, messageId);
			if (summary != null) {
				detail = getBundleString(defBundle, messageId + DETAIL_SUFFIX);
			} else {
				// Try to find detail alone
				detail = getBundleString(appBundle, messageId + DETAIL_SUFFIX);
				if (detail != null) {
					summary = null;
				} else {
					detail = getBundleString(defBundle, messageId
							+ DETAIL_SUFFIX);
					if (detail != null) {
						summary = null;
					} else {
						// Neither detail nor summary found
						facesContext.getExternalContext().log(
								"No message with id " + messageId
										+ " found in any bundle");
						return new FacesMessage(severity, messageId, null);
					}
				}
			}
		}

		if (args != null && args.length > 0) {
			MessageFormat format;

			if (summary != null) {
				format = new MessageFormat(summary, locale);
				summary = format.format(args);
			}

			if (detail != null) {
				format = new MessageFormat(detail, locale);
				detail = format.format(args);
			}
		}

		return new FacesMessage(severity, summary, detail);
	}

	public static String getBundleString(ResourceBundle bundle, String key) {
		try {
			return bundle == null ? null : bundle.getString(key);
		} catch (MissingResourceException e) {
			return null;
		}
	}

	public static ResourceBundle getApplicationBundle(
			FacesContext facesContext, Locale locale) {
		String bundleName = facesContext.getApplication().getMessageBundle();
		if (bundleName != null) {
			return getBundle(facesContext, locale, bundleName);
		} else {
			return null;
		}
	}

	public static ResourceBundle getDefaultBundle(FacesContext facesContext,
			Locale locale) {
		return getBundle(facesContext, locale, FacesMessage.FACES_MESSAGES);
	}

	public static ResourceBundle getBundle(FacesContext facesContext,
			Locale locale, String bundleName) {
		try {
			// First we try the JSF implementation class loader
			return ResourceBundle.getBundle(bundleName, locale, facesContext
					.getClass().getClassLoader());
		} catch (MissingResourceException ignore1) {
			try {
				// Next we try the JSF API class loader
				return ResourceBundle.getBundle(bundleName, locale,
						MessageUtils.class.getClassLoader());
			} catch (MissingResourceException ignore2) {
				try {
					// Last resort is the context class loader
					return ResourceBundle.getBundle(bundleName, locale, Thread
							.currentThread().getContextClassLoader());
				} catch (MissingResourceException damned) {
					facesContext.getExternalContext().log(
							"resource bundle " + bundleName
									+ " could not be found");
					return null;
				}
			}
		}
	}

}
