package org.jvnet.hifaces.function;

import java.io.Serializable;

import javax.faces.context.FacesContext;
import javax.faces.el.EvaluationException;
import javax.faces.el.MethodNotFoundException;

public abstract class Function2<R, P0, P1> extends Function<R> implements
		Serializable {

	public Function2(Class<? extends R> type) {
		super(type);
	}

	@SuppressWarnings("unchecked")
	@Override
	public Object invoke(FacesContext facesContext, Object[] aobj)
			throws EvaluationException, MethodNotFoundException {
		assert aobj.length == 2;
		try {

			final P0 arg0 = (P0) aobj[0];
			final P1 arg1 = (P1) aobj[1];
			return invoke(facesContext, arg0, arg1);
		} catch (Exception ex) {
			throw new EvaluationException(ex);
		}
	}

	public R invoke(FacesContext facesContext, P0 arg0, P1 arg1)
			throws Exception {
		return call(arg0, arg1);
	}

	public R call(P0 arg0, P1 arg1) throws Exception {
		return null;
	}

	public Function1<R, P1> bind0(final P0 arg0) {
		return new Bound0<R, P0, P1>(this, arg0);
	}

	public Function1<R, P0> bind1(final P1 arg1) {
		return new Bound1<R, P0, P1>(this, arg1);
	}

	public Function0<R> bind(final P0 arg0, final P1 arg1) {
		return bind0(arg0).bind(arg1);
	}

	public Function1<R, P1> compose0(final Function0<P0> arg0) {
		return new Composed0<R, P0, P1>(this, arg0);
	}

	public Function1<R, P0> compose1(final Function0<P1> arg1) {
		return new Composed1<R, P0, P1>(this, arg1);
	}

	public Function0<R> compose(final Function0<P0> arg0,
			final Function0<P1> arg1) {
		return compose0(arg0).compose(arg1);
	}

	public static class Bound0<R, P0, P1> extends Function1<R, P1> {
		private static final long serialVersionUID = 1L;

		private final Function2<R, P0, P1> f;

		private final P0 arg0;

		public Bound0(Function2<R, P0, P1> f, P0 arg0) {
			super(f.getType());
			this.f = f;
			this.arg0 = arg0;
		}

		@Override
		public R invoke(FacesContext facesContext, P1 arg1) throws Exception {
			return f.invoke(facesContext, arg0, arg1);
		}

		@Override
		public R call(P1 arg1) throws Exception {
			return f.call(arg0, arg1);
		}
	}

	public static class Bound1<R, P0, P1> extends Function1<R, P0> {
		private static final long serialVersionUID = 1L;

		private final Function2<R, P0, P1> f;

		private final P1 arg1;

		public Bound1(Function2<R, P0, P1> f, P1 arg1) {
			super(f.getType());
			this.f = f;
			this.arg1 = arg1;
		}

		@Override
		public R invoke(FacesContext facesContext, P0 arg0) throws Exception {
			return f.invoke(facesContext, arg0, arg1);
		}

		@Override
		public R call(P0 arg0) throws Exception {
			return f.call(arg0, arg1);
		}
	}

	public static class Composed1<R, P0, P1> extends Function1<R, P0> {
		private static final long serialVersionUID = 1L;

		private final Function2<R, P0, P1> f;

		private final Function0<P1> arg1;

		public Composed1(Function2<R, P0, P1> f, Function0<P1> arg1) {
			super(f.getType());
			this.f = f;
			this.arg1 = arg1;
		}

		@Override
		public R invoke(FacesContext facesContext, P0 arg0) throws Exception {
			return f.invoke(facesContext, arg0, arg1.invoke(facesContext));
		}

		@Override
		public R call(P0 arg0) throws Exception {
			return f.call(arg0, arg1.call());
		}
	}

	public static class Composed0<R, P0, P1> extends Function1<R, P1> {
		private static final long serialVersionUID = 1L;

		private final Function2<R, P0, P1> f;

		private final Function0<P0> arg0;

		public Composed0(Function2<R, P0, P1> f, Function0<P0> arg0) {
			super(f.getType());
			this.f = f;
			this.arg0 = arg0;
		}

		@Override
		public R invoke(FacesContext facesContext, P1 arg1) throws Exception {
			return f.invoke(facesContext, arg0.invoke(facesContext), arg1);
		}

		@Override
		public R call(P1 arg1) throws Exception {
			return f.call(arg0.call(), arg1);
		}
	}

}
