package org.jvnet.hifaces.function;

import java.io.Serializable;

import javax.faces.context.FacesContext;
import javax.faces.el.EvaluationException;
import javax.faces.el.MethodNotFoundException;

public abstract class Function3<R, P0, P1, P2> extends Function<R> implements
		Serializable {

	public Function3(Class<? extends R> type) {
		super(type);
	}

	@SuppressWarnings("unchecked")
	@Override
	public Object invoke(FacesContext facesContext, Object[] aobj)
			throws EvaluationException, MethodNotFoundException {
		assert aobj.length == 3;
		try {
			final P0 arg0 = (P0) aobj[0];
			final P1 arg1 = (P1) aobj[1];
			final P2 arg2 = (P2) aobj[2];
			return invoke(facesContext, arg0, arg1, arg2);
		} catch (Exception ex) {
			throw new EvaluationException(ex);
		}
	}

	public R invoke(FacesContext facesContext, P0 arg0, P1 arg1, P2 arg2)
			throws Exception {
		return call(arg0, arg1, arg2);
	}

	public R call(P0 arg0, P1 arg1, P2 arg2) throws Exception {
		return null;
	}

	public Function2<R, P1, P2> bind0(final P0 arg0) {
		return new Bound0<R, P0, P1, P2>(this, arg0);
	}

	public Function2<R, P0, P2> bind1(final P1 arg1) {
		return new Bound1<R, P0, P1, P2>(this, arg1);
	}

	public Function2<R, P0, P1> bind2(final P2 arg2) {
		return new Bound2<R, P0, P1, P2>(this, arg2);
	}

	public Function0<R> bind(final P0 arg0, final P1 arg1, final P2 arg2) {
		return bind0(arg0).bind0(arg1).bind(arg2);
	}

	public Function2<R, P1, P2> compose0(final Function0<P0> arg0) {
		return new Composed0<R, P0, P1, P2>(this, arg0);
	}

	public Function2<R, P0, P2> compose1(final Function0<P1> arg1) {
		return new Composed1<R, P0, P1, P2>(this, arg1);
	}

	public Function2<R, P0, P1> compose2(final Function0<P2> arg2) {
		return new Composed2<R, P0, P1, P2>(this, arg2);
	}

	public Function0<R> compose(final Function0<P0> arg0,
			final Function0<P1> arg1, final Function0<P2> arg2) {
		return compose0(arg0).compose0(arg1).compose(arg2);
	}

	public static class Bound0<R, P0, P1, P2> extends Function2<R, P1, P2> {
		private static final long serialVersionUID = 1L;

		private final Function3<R, P0, P1, P2> f;

		private final P0 arg0;

		public Bound0(Function3<R, P0, P1, P2> f, P0 arg0) {
			super(f.getType());
			this.f = f;
			this.arg0 = arg0;
		}

		@Override
		public R invoke(FacesContext facesContext, P1 arg1, P2 arg2) throws Exception {
			return f.invoke(facesContext, arg0, arg1, arg2);
		}

		@Override
		public R call(P1 arg1, P2 arg2) throws Exception {
			return f.call(arg0, arg1, arg2);
		}
	}

	public static class Bound1<R, P0, P1, P2> extends Function2<R, P0, P2> {
		private static final long serialVersionUID = 1L;

		private final Function3<R, P0, P1, P2> f;

		private final P1 arg1;

		public Bound1(Function3<R, P0, P1, P2> f, P1 arg1) {
			super(f.getType());
			this.f = f;
			this.arg1 = arg1;
		}

		@Override
		public R invoke(FacesContext facesContext, P0 arg0, P2 arg2) throws Exception {
			return f.invoke(facesContext, arg0, arg1, arg2);
		}

		@Override
		public R call(P0 arg0, P2 arg2) throws Exception {
			return f.call(arg0, arg1, arg2);
		}
	}

	public static class Bound2<R, P0, P1, P2> extends Function2<R, P0, P1> {
		private static final long serialVersionUID = 1L;

		private final Function3<R, P0, P1, P2> f;

		private final P2 arg2;

		public Bound2(Function3<R, P0, P1, P2> f, P2 arg2) {
			super(f.getType());
			this.f = f;
			this.arg2 = arg2;
		}

		@Override
		public R invoke(FacesContext facesContext, P0 arg0, P1 arg1) throws Exception {
			return f.invoke(facesContext, arg0, arg1, arg2);
		}

		@Override
		public R call(P0 arg0, P1 arg1) throws Exception {
			return f.call(arg0, arg1, arg2);
		}
	}

	public static class Composed0<R, P0, P1, P2> extends Function2<R, P1, P2> {
		private static final long serialVersionUID = 1L;

		private final Function3<R, P0, P1, P2> f;

		private final Function0<P0> arg0;

		public Composed0(Function3<R, P0, P1, P2> f, Function0<P0> arg0) {
			super(f.getType());
			this.f = f;
			this.arg0 = arg0;
		}

		@Override
		public R invoke(FacesContext facesContext, P1 arg1, P2 arg2)
				throws Exception {
			return f
					.invoke(facesContext, arg0.invoke(facesContext), arg1, arg2);
		}

		@Override
		public R call(P1 arg1, P2 arg2) throws Exception {
			return f.call(arg0.call(), arg1, arg2);
		}
	}

	public static class Composed1<R, P0, P1, P2> extends Function2<R, P0, P2> {
		private static final long serialVersionUID = 1L;

		private final Function3<R, P0, P1, P2> f;

		private final Function0<P1> arg1;

		public Composed1(Function3<R, P0, P1, P2> f, Function0<P1> arg1) {
			super(f.getType());
			this.f = f;
			this.arg1 = arg1;
		}

		@Override
		public R invoke(FacesContext facesContext, P0 arg0, P2 arg2)
				throws Exception {
			return f
					.invoke(facesContext, arg0, arg1.invoke(facesContext), arg2);
		}

		@Override
		public R call(P0 arg0, P2 arg2) throws Exception {
			return f.call(arg0, arg1.call(), arg2);
		}
	}

	public static class Composed2<R, P0, P1, P2> extends Function2<R, P0, P1> {
		private static final long serialVersionUID = 1L;

		private final Function3<R, P0, P1, P2> f;

		private final Function0<P2> arg2;

		public Composed2(Function3<R, P0, P1, P2> f, Function0<P2> arg2) {
			super(f.getType());
			this.f = f;
			this.arg2 = arg2;
		}

		@Override
		public R invoke(FacesContext facesContext, P0 arg0, P1 arg1)
				throws Exception {
			return f
					.invoke(facesContext, arg0, arg1, arg2.invoke(facesContext));
		}

		@Override
		public R call(P0 arg0, P1 arg1) throws Exception {
			return f.call(arg0, arg1, arg2.call());
		}
	}

}
