package org.jvnet.hifaces.myfaces.custom.tree2;

import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreePath;

import org.apache.myfaces.custom.tree2.TreeModel;
import org.apache.myfaces.custom.tree2.TreeState;
import org.apache.myfaces.custom.tree2.TreeWalker;
import org.jvnet.hifaces.event.TreeModelListenerList;

public abstract class SmartTreeModel implements TreeModel, javax.swing.tree.TreeModel {

	private static final long serialVersionUID = 1L;

	private final TreeModelListenerList treeModelListeners = new TreeModelListenerList();

	private final SmartTreeNode root;

	private final SmartTreeState treeState;

	public SmartTreeModel() {
		this.root = createSmartRoot();
		this.treeState = createTreeState(root);
	}
	
	public abstract SmartTreeNode createSmartRoot();

	public SmartTreeWalker createTreeWalker(SmartTreeNode node) {
		return new SmartTreeWalker(node);
	}

	public SmartTreeState createTreeState(SmartTreeNode node) {
		return new SmartTreeState(node);
	}

	protected SmartTreeNode convert(Object node) {
		return (SmartTreeNode) node;
	}

	protected boolean isCorrectType(Object node) {
		return node instanceof SmartTreeNode;
	}

	public Object getRoot() {
		return root;
	}

	public SmartTreeNode getSmartRoot() {
		return root;
	}

	public SmartTreeNode getFirstLeaf() {
		return getSmartRoot().getFirstLeaf();
	}

	public org.apache.myfaces.custom.tree2.TreeNode getNodeById(String nodeId) {
		return getSmartRoot().getNodeById(nodeId);
	}

	public String[] getPathInformation(String nodeId) {
		final SmartTreeNode node = getSmartRoot().getNodeById(nodeId);
		if (node == null) {
			return new String[0];
		} else {
			return node.getPathInformation();
		}
	}

	public TreeWalker getTreeWalker() {
		return createTreeWalker(getSmartRoot());
	}

	public TreeState getTreeState() {
		return treeState;
	}

	public boolean isLastChild(String nodeId) {

		final SmartTreeNode treeNode = getSmartRoot().getNodeById(nodeId);
		final SmartTreeNode parentNode = treeNode.getSmartParent();
		if (parentNode == null) {
			return true;
		} else {
			return treeNode.isLastChild();
		}
	}

	public void setTreeState(TreeState state) {
		// TODO but not now
	}

	public Object getChild(Object parent, int index) {
		if (!isCorrectType(parent))
			throw new IllegalArgumentException("Wrong node type.");
		return convert(parent).getChildAt(index);
	}

	public int getChildCount(Object parent) {
		if (!isCorrectType(parent))
			throw new IllegalArgumentException("Wrong node type.");
		return convert(parent).getChildCount();
	}

	public int getIndexOfChild(Object parent, Object child) {
		if (!isCorrectType(parent))
			throw new IllegalArgumentException("Wrong node type.");
		return convert(parent).getIndex(convert(child));
	}

	// {
	// return root;
	// }

	public boolean isLeaf(Object node) {
		if (!isCorrectType(node))
			throw new IllegalArgumentException("Wrong node type.");
		return convert(node).isLeaf();
	}

	public void insertNodeInto(MutableTreeNode newChild,
			MutableTreeNode draftParent, int index) {
		if (!isCorrectType(newChild))
			throw new IllegalArgumentException("Wrong node type.");
		if (!isCorrectType(draftParent))
			throw new IllegalArgumentException("Wrong node type.");
		final SmartTreeNode parent = convert(draftParent);
		parent.insert(convert(newChild), index);

		int[] newIndexs = new int[1];

		newIndexs[0] = index;
		nodesWereInserted(parent, newIndexs);
	}

	public void addNodeTo(MutableTreeNode newChild, MutableTreeNode draftParent) {
		insertNodeInto(newChild, draftParent, draftParent.getChildCount());
	}

	public void removeNodeFromParent(MutableTreeNode draftNode) {
		if (!isCorrectType(draftNode))
			throw new IllegalArgumentException("Wrong node type.");
		SmartTreeNode node = convert(draftNode);
		SmartTreeNode parent = convert(node.getParent());

		if (parent == null)
			throw new IllegalArgumentException("node does not have a parent.");

		int[] childIndex = new int[1];
		Object[] removedArray = new Object[1];

		childIndex[0] = parent.getIndex(node);
		parent.remove(childIndex[0]);
		removedArray[0] = node;
		nodesWereRemoved(parent, childIndex, removedArray);
	}

	public void nodesWereInserted(SmartTreeNode node, int[] childIndices) {
		if (node != null && childIndices != null && childIndices.length > 0) {
			int cCount = childIndices.length;
			Object[] newChildren = new Object[cCount];

			for (int counter = 0; counter < cCount; counter++)
				newChildren[counter] = node.getChildAt(childIndices[counter]);
			treeModelListeners.fireTreeNodesInserted(new TreeModelEvent(this,
					getPathToRoot(node), childIndices, newChildren));
		}
	}

	public void nodesWereRemoved(SmartTreeNode node, int[] childIndices,
			Object[] removedChildren) {
		if (node != null && childIndices != null) {
			treeModelListeners.fireTreeNodesRemoved(new TreeModelEvent(this,
					getPathToRoot(node), childIndices, removedChildren));
		}
	}

	public void nodeChanged(SmartTreeNode node) {
		if (treeModelListeners != null && node != null) {
			SmartTreeNode parent = convert(node.getParent());

			if (parent != null) {
				int anIndex = parent.getIndex(node);
				if (anIndex != -1) {
					int[] cIndexs = new int[1];

					cIndexs[0] = anIndex;
					nodesChanged(parent, cIndexs);
				}
			} else if (node == getRoot()) {
				nodesChanged(node, null);
			}
		}
	}

	public void nodesChanged(SmartTreeNode node, int[] childIndices) {
		if (node != null) {
			if (childIndices != null) {
				int cCount = childIndices.length;

				if (cCount > 0) {
					Object[] cChildren = new Object[cCount];

					for (int counter = 0; counter < cCount; counter++)
						cChildren[counter] = node
								.getChildAt(childIndices[counter]);
					treeModelListeners
							.fireTreeNodesChanged(new TreeModelEvent(this,
									getPathToRoot(node), childIndices,
									cChildren));
				}
			} else if (node == getRoot()) {
				treeModelListeners.fireTreeNodesChanged(new TreeModelEvent(
						this, getPathToRoot(node), null, null));
			}
		}
	}

	public Object[] getPathToRoot(SmartTreeNode aNode) {
		return getPathToRoot(aNode, 0);
	}

	protected Object[] getPathToRoot(SmartTreeNode aNode, int depth) {
		Object[] retNodes;
		if (aNode == null) {
			if (depth == 0)
				return null;
			else
				retNodes = new Object[depth];
			// java.lang.reflect.Array.newInstance(aNode
			// .getClass(), depth);

		} else {
			depth++;
			if (aNode == getRoot())
				retNodes = new Object[depth];
			else
				retNodes = getPathToRoot(aNode.getSmartParent(), depth);
			retNodes[retNodes.length - depth] = aNode;
		}
		return retNodes;
	}

	public void valueForPathChanged(TreePath path, Object newValue) {
		throw new UnsupportedOperationException();
	}

	public void addTreeModelListener(TreeModelListener l) {
		treeModelListeners.addChangeListener(l);
	}

	public void removeTreeModelListener(TreeModelListener l) {
		treeModelListeners.removeChangeListener(l);
	}

}
