package org.jvnet.hifaces.util;

import java.util.Iterator;

import javax.faces.component.UIComponent;
import javax.faces.component.UIViewRoot;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Faces utilities.
 * @author valikov
 */
public class DebugUtils {

  /**
   * Hidden constructor.
   */
  private DebugUtils() {
  }

  /**
   * Logger.
   */
  protected static Log log = LogFactory.getLog(DebugUtils.class);

  /**
   * Returns view root of the component.
   * @param component target component.
   * @return View root of the component.
   */
  public static UIViewRoot getViewRoot(UIComponent component) {
    if (component instanceof UIViewRoot) {
      return (UIViewRoot) component;
    }
    else if (component == null) {
      return null;
    }
    else {
      return getViewRoot(component.getParent());
    }
  }

  public static UIComponent getRoot(UIComponent component) {
    if (component.getParent() == null)
      return component;
    else
      return getRoot(component.getParent());
  }
  
  public static UIComponent findComponent(UIComponent component, String id) {
    if (id.equals(component.getId()))
      return component;
    else
      if (component.getChildCount() > 0) {
        for (Iterator iterator = component.getChildren().iterator(); iterator.hasNext();) {
          final UIComponent found = findComponent((UIComponent) iterator.next(), id);
          if (found !=null)
            return found;
        }
      }
      return null;
  }
  

  /**
   * Creates a unique id for the component.
   * @param component target component. 
   * @return Unique id for the component.
   */
  public static String createUniqueId(UIComponent component) {
    int id = 0;
//    final UIComponent root = getRoot(component);
//    debugIds(root);
    while (component.findComponent(/*root,*/ "hid" + id) != null) {
//    while (findComponent(root, "hid" + id) != null) {
      id++;
    }
    return "hid" + id;
  }

  /**
   * Assigns unique id to the component.
   * @param component target component.
   */
  public static void setUniqueId(UIComponent component) {
    final String id = createUniqueId(component);
    component.setId(id);
  }

  /**
   * Outputs ids of the components.
   * @param component component to start debugging at.
   */
  public static void debugIds(UIComponent component) {
    debugIds(component, "");
  }

  public static void debugIds(UIComponent component, String ident) {
    log.debug(ident
        + "Id ["
        + component.getId()
        + "] component ["
        + component
        + "] of class ["
        + component
        + "].");
    if (component.getChildCount() > 0) {
      for (Iterator iterator = component.getChildren().iterator(); iterator.hasNext();) {
        debugIds((UIComponent) iterator.next(), ident + "  ");
      }
    }
  }

  public static void setUniqueIds(UIComponent component) {
    setUniqueId(component);
    if (component.getChildCount() > 0) {
      for (Iterator iterator = component.getChildren().iterator(); iterator.hasNext();) {
        setUniqueIds((UIComponent) iterator.next());
      }
    }
  }
}
